﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace MathParser
{
    
    public static class Extensions
    {
        public static string[] GetDescriptions(this Enum value)
        {
            FieldInfo fi = value.GetType().GetField(value.ToString());
            DescriptionAttribute[] attributes =
                (DescriptionAttribute[])fi.GetCustomAttributes(
                typeof(DescriptionAttribute),
                false);

            if (attributes != null &&
                attributes.Length > 0)
                return attributes[0].Description;
            else
                return null;
        }
        public static IEnumerable<T> GetTypesWithPriority<T>(this int pPriority)
        {
            FieldInfo[] fields = typeof(T).GetFields();
            foreach (var item in fields)
            {
                foreach (var elems in item.GetCustomAttributes(typeof(DescriptionAttribute), false))
                {
                    if (((DescriptionAttribute)elems).Priority == pPriority)
                    {
                        yield return (T)item.GetValue(null);
                    }
                }
            }
        }
        public static T GetEnumFromDescription<T>(this string description)
        {
            var type = typeof(T);
            if (!type.IsEnum)
                throw new ArgumentException();
            FieldInfo[] fields = type.GetFields();
            var field = fields
                            .SelectMany(f => f.GetCustomAttributes(
                                typeof(DescriptionAttribute), false), (
                                    f, a) => new { Field = f, Att = a })
                            .Where(a => ((DescriptionAttribute)a.Att)
                                .Description.Any((item) => item == description)).SingleOrDefault();
            return field == null ? default(T) : (T)field.Field.GetRawConstantValue();
        }
        public static int GetOperators(this Enum value)
        {
            FieldInfo fi = value.GetType().GetField(value.ToString());
            DescriptionAttribute[] attributes =
                (DescriptionAttribute[])fi.GetCustomAttributes(
                typeof(DescriptionAttribute),
                false);

            if (attributes != null &&
                attributes.Length > 0)
                return attributes[0].Operators;
            else
                return 0;
        }
        public static string GetDescriptions(this System.Type type)
        {
            string lInput = string.Empty;
            FieldInfo[] fields = type.GetFields();
            foreach (var item in fields)
            {
                foreach (var elems in item.GetCustomAttributes(typeof(DescriptionAttribute), false))
                {
                    lInput += string.Concat(((DescriptionAttribute)elems).Description);
                }
            }
            return lInput;
        }
    }
}
