﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace MathParser
{
    public partial class frmMain : Form
    {
        private Parser _parser;
        public frmMain()
        {
            InitializeComponent();
            _parser = new Parser();
            Symboltable.AddID("PI", Math.PI, true);
            Symboltable.AddID("E", Math.E, true);
            Symboltable.AddID("TRUE", 1, true);
            Symboltable.AddID("FALSE", 0, true);
            Function.AddFunction("sqrt", 1, (value) => Math.Sqrt(value[0]));
            Function.AddFunction("log", 2, (value) => Math.Log(value[0], value[1]));
            Function.AddFunction("log10", 1, (value) => Math.Log10(value[0]));
            Function.AddFunction("sin", 1, (value) => Math.Sin(value[0]));
            Function.AddFunction("cos", 1, (value) => Math.Cos(value[0]));
            Function.AddFunction("tan", 1, (value) => Math.Tan(value[0]));
            Function.AddFunction("equals", 2, (value) =>
                {
                    if (value[0] == value[1])
                        return 1;
                    return 0;
                });
            Function.AddFunction("xor", 2, (value) => (int)value[0] ^ (int)value[1]);
            Function.AddFunction("and", 2, (value) => (int)value[0] & (int)value[1]);
            Function.AddFunction("or", 2, (value) => (int)value[0] | (int)value[1]);
            Function.AddFunction("neg", 1, (value) => ~(int)value[0]);
            Function.AddFunction("pow", 2, (value) => Math.Pow(value[0], value[1]));
        }
        private void Fill(SyntaxTree pTree, TreeNode pNode)
        {
            TreeNode lNode = null;
            var lDescs = pTree.TreeType.GetDescriptions();
            if (pTree.TreeType == Type.ID && lDescs != null)
                lNode = new TreeNode(lDescs[0] + " (" + pTree.Description + ")");
            else
                lNode = new TreeNode(pTree.TreeType.ToString() + " (" + pTree.Value + ")");
            pNode.Nodes.Add(lNode);
            foreach (var item in pTree.Nodes)
            {
                Fill(item, lNode);
            }
        }
        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                fypViews.Controls.Clear();
                foreach (var ite in rtbTerm.Text.Split(';'))
                {
                    if (string.IsNullOrEmpty(ite))
                        continue;
                    TreeView tVTree = new TreeView();
                    tVTree.Height = 150;
                    tVTree.Width = 230;
                    tVTree.Nodes.Clear();
                    Token[] allToken = Scanner.StartScan(ite).ToArray();
                    SyntaxTree lErgebniss = _parser.Parse(allToken);
                    lblErgebniss.Text = lErgebniss.Evaluate().ToString();
                    TreeNode lNode = null;
                    var lDescs = lErgebniss.TreeType.GetDescriptions();
                    if (lErgebniss.TreeType == Type.ID && lDescs != null)
                        lNode = new TreeNode(lDescs[0] + " (" + lErgebniss.Description + ")");
                    else
                        lNode = new TreeNode(lErgebniss.TreeType.ToString() + " (" + lErgebniss.Value + ")");
                    foreach (var item in lErgebniss.Nodes)
                    {
                        Fill(item, lNode);
                    }
                    tVTree.Nodes.Add(lNode);
                    tVTree.ExpandAll();
                    fypViews.Controls.Add(tVTree);
                }
            } catch (ArgumentException ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            Symboltable.Clear();
        }
    }

}
