﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
namespace MathParser
{
    public class Parser
    {
        private Queue<Token> allTokens;
        private Token lToken;
        private SyntaxTree start()
        {
            return NextPriority(1);
        }
        private void accept(Type pType)
        {
            if (lToken.TokenType != pType)
                throw new ArgumentException("Syntaxfehler an Position " + (_max - allTokens.Count) + " (gefunden: '" + lToken.Value + "' vorgesehen: '" + pType.ToString() + "')");
            getNextToken();
        }
        private void getNextToken()
        {
            if (lToken == null || lToken.TokenType != Type.EOF)
            lToken = allTokens.Dequeue();
        }
        private SyntaxTree NextPriority(int pPriority)
        {
            Type[] lTypes = pPriority.GetTypesWithPriority<Type>().ToArray();
            if (!lTypes.Any())
                return null;
            SyntaxTree res = NextPriority(pPriority + 1);
            if (res == null)
                res = klammer();
            var lItem = lTypes.Where((elem) => elem == lToken.TokenType);
            while (lItem.Any())
            {
                Type lType = lItem.First();
                int lOperators = lType.GetOperators();
                getNextToken();
                        res = new SyntaxTree(0, lType, res);
                        for (int i = 1; i < lOperators; i++)
                        {
                            var lPriority = NextPriority(pPriority + 1);
                            if (lPriority == null)
                                res.Nodes.Add(klammer());
                            else
                                res.Nodes.Add(lPriority);
                        }
                lItem = lTypes.Where((elem) => elem == lToken.TokenType);
            }
            return res;
        }
        private SyntaxTree klammer()
        {
            if (lToken.TokenType == Type.NEXTPARAM)
                accept(Type.NEXTPARAM);
            int sign = 1;
            SyntaxTree lValue = null;
            if (lToken.TokenType == Type.ADD || lToken.TokenType == Type.SUB)
            {
                if (lToken.TokenType == Type.SUB)
                    sign = -1;
                getNextToken();
            }
            if (lToken.TokenType == Type.ID)
            {
                    string lName = string.Empty;
                    lName = lToken.Value;
                    accept(Type.ID);
                if (lToken.TokenType == Type.BROPEN)
                {
                    accept(Type.BROPEN);
                    List<SyntaxTree> allParams = new List<SyntaxTree>();
                    allParams.Add(start());
                    while (lToken.TokenType == Type.NEXTPARAM)
                        allParams.Add(start());
                    lValue = new SyntaxTree(0, Type.FUNC, lName, allParams.ToArray());
                    accept(Type.BRCLOSE);
                }
                else
                    lValue = new SyntaxTree(0, Type.ID, lName);
            }
            else if (lToken.TokenType == Type.BROPEN)
            {
                accept(Type.BROPEN);
                lValue = new SyntaxTree(0, Type.MUL, start(), new SyntaxTree(sign, Type.INTNUM));
                accept(Type.BRCLOSE);
            }
            else if (lToken.TokenType == Type.DECNUM || lToken.TokenType == Type.INTNUM)
            {
                lValue = new SyntaxTree(double.Parse(lToken.Value, CultureInfo.InvariantCulture) * sign, lToken.TokenType);
                accept(lToken.TokenType);
            }
            else if (lToken.TokenType != Type.NEXTPARAM)
                throw new ArgumentException("Syntaxfehler an Position " + (_max - allTokens.Count) + " (gefunden: '" + lToken.Value + "')");
            return lValue;
        }
        private int _max;
        public SyntaxTree Parse(IEnumerable<Token> pTokens)
        {
            _max = pTokens.Count();
            lToken = null;
            allTokens = new Queue<Token>(pTokens);
            getNextToken();
            SyntaxTree res = start();
            accept(Type.EOF);
            return res;
        }
    }
}
