﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MathParser
{
    public static class Scanner
    {
        private static Automat _automat;
        static Scanner()
        {
            Zustand lDecimal = new Zustand(true, "0123456789".ToCharArray());
            lDecimal.NaechsteZustaende.Add(lDecimal);
            Zustand DecimalComma = new Zustand(false, ".".ToCharArray(), lDecimal);
            Zustand lNumber = new Zustand(true, "0123456789".ToCharArray());
            lNumber.NaechsteZustaende.Add(lNumber);
            lNumber.NaechsteZustaende.Add(DecimalComma);
            Zustand lOperator = new Zustand(true, typeof(Type).GetDescriptions().ToCharArray());
            Zustand lBezeichner = new Zustand(true, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".ToCharArray());
            lBezeichner.NaechsteZustaende.Add(lBezeichner);
            lBezeichner.NaechsteZustaende.Add(lDecimal);
            Zustand lStart = new Zustand(false, new char[] { }, lNumber, lOperator, lBezeichner);
            _automat = new Automat(lStart);
        }
        private static IEnumerable<LinedChar> getChars(string pFormula)
        {
            int lRowIndex = 0;
            int lColumnIndex = 0;
            bool lGet = true;
            for (int i = 0; i < pFormula.Length; i++)
            {
                int lIndex = i;
                char lChar = pFormula[i];
                if (lChar == '\n')
                {
                    lColumnIndex = 0;
                    lRowIndex++;
                    lGet = true;
                }
                else if (lChar == '\'')
                    lGet = false;
                else if (lChar == ' ' || !lGet)
                    continue;
                else
                {
                    yield return new LinedChar(lChar, lIndex, lRowIndex, lColumnIndex);
                    lColumnIndex++;
                }
             
            }
        }
        public static IEnumerable<Token> StartScan(string pFormula)
        {
            Token lToken = null;
            var lElements = getChars(pFormula).ToArray();
            for (int item = 0; item < lElements.Length; item++)
            {
                if (getToken(lElements[item], ref lToken) == AutomatenZustand.Ende)
                {
                    yield return lToken;
                    item--;
                }
                else
                    continue;
                lToken = null;
            }
            var token = getToken(' ', ref lToken);
            if (token == AutomatenZustand.Ende)
                yield return lToken;
            else if (token == AutomatenZustand.Fehler)
                ErrorScanning("Invalid Symbol found: " + _automat.ToString() + " at last Index");
            yield return new Token("", Type.EOF);
        }
        private static void ErrorScanning(string pError)
        {
            throw new ArgumentException(pError);
        }
        private static AutomatenZustand getToken(LinedChar pChar, ref Token pToken)
        {
            AutomatenZustand lZustand = getToken(pChar.Value, ref pToken);
            if (lZustand == AutomatenZustand.Fehler)
                ErrorScanning("Invalid Symbol found: " + _automat.ToString() + " at Index: " + (pChar.Index + 1) + " in Line: " + (pChar.RowIndex + 1));
            return lZustand;
        }
        private static AutomatenZustand getToken(char pChar, ref Token pToken)
        {
            AutomatenZustand lZustand = _automat.NaechsterZustand(pChar);
            if (lZustand == AutomatenZustand.Ende)
            {
                string lToken = _automat.ToString();

                int lResult = 0;
                decimal lRes = 0;
                if (int.TryParse(lToken, out lResult))
                    pToken = new Token(lToken, Type.INTNUM);
                else if (decimal.TryParse(lToken, out lRes))
                    pToken = new Token(lToken, Type.DECNUM);
                else if (!lToken.Any((elem) => typeof(Type).GetDescriptions().Contains(elem)))
                    pToken = new Token(lToken, Type.ID);
                else
                    pToken = new Token(lToken, lToken.GetEnumFromDescription<Type>());
            }
            return lZustand;
        }
    }
}
