﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MathParser
{
    public class SyntaxTree
    {
        public SyntaxTree(double pValue, Type pType,  params SyntaxTree[] pNodes)
        {
            this.Value = pValue;
            this.TreeType = pType;
            this.Nodes = pNodes.ToList();
        }
        public SyntaxTree(double pValue, Type pType, string pDescription, params SyntaxTree[] pNodes)
        {
            this.Value = pValue;
            this.TreeType = pType;
            this.Nodes = pNodes.ToList();
            this.Description = pDescription;
        }
        public double Value { get; private set; }
        public Type TreeType { get; private set; }
        public string Description { get; private set; }
        public List<SyntaxTree> Nodes { get; private set; }
        public double Evaluate()
        {
            //if (Value != 0)
            //    return Value;
            double lValue = 0;
            switch (TreeType)
            {
                case Type.ADD:
                    lValue = (Nodes[0].Evaluate() + Nodes[1].Evaluate());
                    break;
                case Type.SUB:
                    lValue = (Nodes[0].Evaluate() - Nodes[1].Evaluate());
                    break;
                case Type.MUL:
                    lValue = (Nodes[0].Evaluate() * Nodes[1].Evaluate());
                    break;
                case Type.DIV:
                    lValue = (Nodes[0].Evaluate() / Nodes[1].Evaluate());
                    break;
                case Type.INTDIV:
                    lValue = (int)(Nodes[0].Evaluate() / Nodes[1].Evaluate());
                    break;
                case Type.MODULO:
                    lValue = (Nodes[0].Evaluate() % Nodes[1].Evaluate());
                    break;
                case Type.FAKULTAET:
                    int lResult = 1;
                    for (int i = 2; i <= Nodes[0].Evaluate(); i++)
                        lResult *= i;
                    lValue = lResult;
                    break;
                case Type.FUNC:
                    lValue = Function.Execute(this.Description, Nodes.Select((elem) => elem.Evaluate()).ToArray());
                    break;
                case Type.ID:
                    lValue = Symboltable.GetID(Description);
                    break;
                case Type.HIGHER:
                    if (Nodes[0].Evaluate() > Nodes[1].Evaluate())
                        lValue = 1;
                    else
                        lValue = 0;
                    break;
                case Type.FOR:
                    for (int i = 0; i < Nodes[0].Evaluate(); i++)
                    {
                        lValue = Nodes[1].Evaluate();
                    }
                        break;
                case Type.LESS:
                     if (Nodes[0].Evaluate() < Nodes[1].Evaluate())
                        lValue = 1;
                    else
                        lValue = 0;
                    break;
                case Type.ASSIGN:
                    if (Nodes[0].TreeType == Type.ID)
                    {
                        double lR = Nodes[1].Evaluate();
                        if (Symboltable.AddID(Nodes[0].Description, lR, false))
                            lValue = lR;
                    }
                    else
                        lValue = 0;
                    break;
                case Type.CONDITION:
                    if (Nodes[0].Evaluate() == 1)
                        lValue = Nodes[1].Evaluate();
                    break;
                case Type.UNTIL:
                    if (Value != 0)
                        lValue = Value;
                    else
                    while (Nodes[1].Evaluate() != 1)
                    {
                        lValue = Nodes[0].Evaluate();
                    }
                    break;
                case Type.ELSECONDITION:
                    if (Nodes[0].Evaluate() == 0)
                        lValue = Nodes[1].Evaluate();
                    else
                        lValue = Nodes[0].Nodes[1].Evaluate();
                    break;
                case Type.XOR:
                    lValue = Function.Execute("xor", Nodes[0].Evaluate(), Nodes[1].Evaluate());
                    break;
                case Type.AND:
                    lValue = Function.Execute("and", Nodes[0].Evaluate(), Nodes[1].Evaluate());
                    break;
                case Type.OR:
                    lValue = Function.Execute("or", Nodes[0].Evaluate(), Nodes[1].Evaluate());
                    break;
                case Type.NEG:
                    lValue = Function.Execute("neg", Nodes[0].Evaluate());
                    break;
                default:
                    return Value;
            }
            Value = lValue;
            return Value;
        }
    }
    
}
